@@ -799,7 +799,7 @@ gst_date_time_new_from_iso8601_string (const gchar * string)
       month = day = -1;
     }
 
-    if (ret >= 1 && year <= 0)
+    if (ret >= 1 && (year <= 0 || year > 9999 || month > 12 || day > 31))
       return NULL;
 
     else if (ret >= 1 && len < 16)
@@ -865,7 +865,7 @@ gst_date_time_new_from_iso8601_string (const gchar * string)
     else if (neg_pos)
       pos = neg_pos + 1;
 
-    if (pos) {
+    if (pos && strlen (pos) >= 3) {
       gint ret_tz;
       if (pos[2] == ':')
         ret_tz = sscanf (pos, "%d:%d", &gmt_offset_hour, &gmt_offset_min);
