@@ -24,6 +24,7 @@
 
  /**
  * SECTION:gsttriggercontrolsource
+ * @title: GstTriggerControlSource
  * @short_description: trigger control source
  *
  * #GstTriggerControlSource is a #GstControlSource, that returns values from user-given
@@ -187,7 +188,7 @@ G_DEFINE_TYPE_WITH_CODE (GstTriggerControlSource, gst_trigger_control_source,
 GstControlSource *
 gst_trigger_control_source_new (void)
 {
-  return g_object_newv (GST_TYPE_TRIGGER_CONTROL_SOURCE, 0, NULL);
+  return g_object_new (GST_TYPE_TRIGGER_CONTROL_SOURCE, NULL);
 }
 
 static void
